library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;
use IEEE.std_logic_arith.all;

entity TPU is
    port( CLK    : in  std_logic;
          RST    : in  std_logic;
          INPUT  : in  std_logic_vector(7 downto 0);
          OUTPUT : out std_logic_vector(7 downto 0);
          OPCODE : in  std_logic_vector(3 downto 0);
          OPCS   : in  std_logic;
          OPACK  : out std_logic);
end TPU;

architecture RTL of TPU is
    signal A : std_logic_vector(7 downto 0);
    signal B : std_logic_vector(7 downto 0);
    signal F : std_logic_vector(7 downto 0);
    type FSMSTATE is (S_IDLE, S_NOP, S_DIN, S_OUT, S_MVB, S_MVA, S_MVF,
                      S_ADD, S_DIV1, S_DIV2, S_DIV3, S_SWP, S_EOS);
    signal STATE  : FSMSTATE;
    signal VA     : integer range 0 to 255;
    signal VB     : integer range 0 to 255;
    signal VANS   : integer range 0 to 255;
begin
    process(CLK, RST)
    variable TMP  : std_logic_vector(7 downto 0);
    begin
        if (RST = '0') then
            STATE <= S_IDLE;
            A <= "00000000";
            B <= "00000000";
            OUTPUT <= "00000000";
            OPACK  <= '0';
            VA <= 0;
            VB <= 0;
            VANS <= 0;
        elsif (CLK'event and CLK = '1') then
            OPACK <= '0';
            case STATE is
            when S_IDLE =>
                if (OPCS = '1') then
                    case OPCODE is
                        when "0000" => STATE <= S_NOP;
                        when "0001" => STATE <= S_DIN;
                        when "0010" => STATE <= S_OUT;
                        when "0011" => STATE <= S_MVB;
                        when "0100" => STATE <= S_MVA;
                        when "0101" => STATE <= S_MVF;
                        when "0110" => STATE <= S_ADD;
                        when "0111" => STATE <= S_DIV1;
                        when "1000" => STATE <= S_SWP;
                        when others => STATE <= S_EOS;
                    end case;
                end if;
            when S_NOP =>
                STATE <= S_EOS;
            when S_DIN =>
                A <= INPUT;
                STATE <= S_EOS;
            when S_OUT =>
                OUTPUT <= A;
                STATE <= S_EOS;
            when S_MVB =>
                B <= A;
                STATE <= S_EOS;
            when S_MVA =>
                A <= B;
                STATE <= S_EOS;
            when S_MVF =>
                A <= F;
                STATE <= S_EOS;
            when S_ADD =>
                A <= A + B;
                STATE <= S_EOS;
            when S_DIV1 =>
                VA <= CONV_INTEGER(A);
                VB <= CONV_INTEGER(B);
                STATE <= S_DIV2;
            when S_DIV2 =>
                VANS <= VA / VB;
                STATE <= S_DIV3;
            when S_DIV3 =>
                A <= CONV_STD_LOGIC_VECTOR(VANS, 8);
                STATE <= S_EOS;
            when S_SWP =>
                TMP := A;
                A <= B;
                B <= TMP;
                STATE <= S_EOS;
            when S_EOS =>
                OPACK <= '1';
                STATE <= S_IDLE;
            when others =>
                STATE <= S_IDLE;
            end case;
        end if;
    end process;

    F <= "11111111" when A > B else "00000000";

end RTL;
