library IEEE,STD;
use ieee.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;
use std.TEXTIO.all;
use ieee.std_logic_textio.all;

entity OPINF is
    port( CLK    : in  std_logic;
          RST    : in  std_logic;
          INPUT  : out std_logic_vector(7 downto 0);
          OUTPUT : in  std_logic_vector(7 downto 0);
          OPCODE : out std_logic_vector(3 downto 0);
          OPCS   : out std_logic;
          OPACK  : in  std_logic);
end OPINF;

architecture SIM of OPINF is
    file INFILE     : text is in  "cmd.txt";
    file OUTFILE    : text is out "log.txt";
    type SIMSTATE is (S_START, S_WAIT, S_END);
    signal STATE    : SIMSTATE;
    signal I_INPUT  : std_logic_vector(7 downto 0);
    signal I_OPCODE : std_logic_vector(3 downto 0);
    signal I_OPCS   : std_logic;
    signal I_CHKDAT : std_logic_vector(7 downto 0);
begin
    process(RST, CLK)
    variable LI   : line;
    variable LO   : line;
    variable CMD  : string(1 to 3);
    variable DAT8 : std_logic_vector(7 downto 0);
    variable BLNK : character      := ' ';
    variable OK   : string(1 to 2) := "OK";
    variable GOOD : string(1 to 4) := "GOOD";
    variable NG   : string(1 to 2) := "NG";
    begin
        if (RST = '0') then
            STATE    <= S_START;
            I_INPUT  <= "00000000";
            I_OPCODE <= "0000";
            I_OPCS   <= '0';
        elsif (CLK'event and CLK = '1') then
            I_OPCS <= '0';
            case STATE is
            when S_START =>
                if (endfile(INFILE)) then
                    assert false report "END" severity FAILURE;
                else
                    readline(INFILE, LI);
                    read(LI, CMD);
                    write(LO, CMD);
                    if (CMD = "INP") then
                        hread(LI, DAT8);
                        write(LO, BLNK);
                        hwrite(LO, DAT8);
                        I_INPUT <= DAT8;
                        STATE <= S_END;
                    elsif (CMD = "CHK") then
                        hread(LI, DAT8);
                        write(LO, BLNK);
                        hwrite(LO, DAT8);
                        I_CHKDAT <= DAT8;
                        STATE <= S_END;
                    else
                        case CMD is
                        when "NOP"  => I_OPCODE <= "0000";
                        when "DIN"  => I_OPCODE <= "0001";
                        when "OUT"  => I_OPCODE <= "0010";
                        when "MVB"  => I_OPCODE <= "0011";
                        when "MVA"  => I_OPCODE <= "0100";
                        when "MVF"  => I_OPCODE <= "0101";
                        when "ADD"  => I_OPCODE <= "0110";
                        when "DIV"  => I_OPCODE <= "0111";
                        when "SWP"  => I_OPCODE <= "1000";
                        when others => I_OPCODE <= "0000";
                        end case;
                        I_OPCS <= '1';
                        STATE <= S_WAIT;
                    end if;
                    writeline(OUTFILE, LO);
                end if;
            when S_WAIT =>
                if (OPACK = '1') then
                    STATE <= S_END;
                end if;
            when S_END =>
                write(LO, BLNK);
                write(LO, OK);
                if (CMD = "OUT") then
                    write(LO, BLNK);
                    hwrite(LO, OUTPUT);
                elsif (CMD = "CHK") then
                    write(LO, BLNK);
                    hwrite(LO, OUTPUT);
                    write(LO, BLNK);
                    hwrite(LO, I_CHKDAT);
                    write(LO, BLNK);
                    if (OUTPUT = I_CHKDAT) then
                        write(LO, GOOD);
                    else
                        write(LO, NG);
                    end if;
                end if;
                writeline(OUTFILE, LO);
                STATE <= S_START;
            end case;
        end if;
    end process;

    INPUT  <= I_INPUT;
    OPCODE <= I_OPCODE;
    OPCS   <= I_OPCS;
end SIM;
